--[[This script either searches the entire current track
or the currently selected group for phonemes specified
by the user and replaces them with other phonemes
that can also be freely selected.
]]

SCRIPT_TITLE = "Find & Replace Phonemes"

function getClientInfo()
  return {
    name = "ak "..SCRIPT_TITLE,
    category = "akStudio",
    author = "akStudio GG",
    versionNumber = 1,
    minEditorVersion = 65537
  }
end

function replacePhonemes(groupRef, input)
  local group = groupRef:getTarget()
  local groupPhonemes = {}
  local noteCount = group:getNumNotes()
  local note = {}
  local oldPhoneme = ""
  local newPhoneme = ""
  local findString = " " .. input.tbFind .. " "
  local replString = " " .. input.tbReplace .. " "
  local replCountNote = 0
  local replCountGroup = 0
  
  groupPhonemes = SV:getPhonemesForGroup(groupRef)
  
  --[[ Copies of groups on the track initially do not have their own phoneme arrays.
       groupPhenomes in this case has type number and would cause an error.]]
  if type(groupPhonemes) ~= "table" then 
    return 0
    
  else
    -- Normally the number of notes in a group should match the phoneme array length.
    -- SV:showMessageBox("Test", #groupPhonemes .. ", " .. noteCount)
    if #groupPhonemes < noteCount then
      return 0
      
    else
      for noteIx = 1, noteCount do
        note = group:getNote(noteIx)
        --[[ So that only complete phonemes can be searched for and replaced,
             they are temporarily marked with spaces.]]
        oldPhoneme = " " .. groupPhonemes[noteIx] .. " "
        
        if string.find(oldPhoneme,findString) then
          newPhoneme, replCountNote = string.gsub(oldPhoneme, findString, replString)
          -- remove spaces at the begin and the end
          note:setPhonemes(string.sub(newPhoneme,2,-2))
          -- The replacements made are counted for control purposes
          replCountGroup = replCountGroup + replCountNote
        end
      end
    end
  end
    
  return replCountGroup
end

function main()
  local mainEditor = SV:getMainEditor()
  local currentTrack = mainEditor:getCurrentTrack()
  local groupCount = currentTrack:getNumGroups()
  local noteGroupRef = {}
  local replCountTrack = 0

  local form = {
    title = SCRIPT_TITLE,
    message = "",
    buttons = "OkCancel",
    widgets = {
      {
        name = "tbFind",
        type = "TextBox",
        label = "Find phonemes...",
        default = "hh"
      },
      {
        name = "tbReplace",
        type = "TextBox",
        label = "Replace with ...",
        default = "br"
      },
      {
        name = "cbOnlyGroup",
        type = "CheckBox",
        text = "replace phonemes only for selected group",
        default = false
      }
    }
  }

  local result = SV:showCustomDialog(form)
  if result.status then
    -- Decision: group or track
    if result.answers.cbOnlyGroup == true then
      noteGroupRef = mainEditor:getCurrentGroup()
      replCountTrack = replacePhonemes(noteGroupRef, result.answers)
    else
       for groupIx = 1, groupCount do
        noteGroupRef = currentTrack:getGroupReference(groupIx)
        replCountTrack = replCountTrack + replacePhonemes(noteGroupRef, result.answers)
      end
    end
  end
  
  SV:showMessageBox("Notice", "Number of replaced phonemes: " .. replCountTrack)
  SV:finish()
end
